@@ -18,6 +18,19 @@ def set_run_action_arguments(parser: argparse.ArgumentParser) -> None:
     sdk.add_endpoint_arguments(parser)
 
 
+def add_interface_argument(
+    parser: Union[argparse.ArgumentParser, argparse._ArgumentGroup]
+) -> None:
+    """Binds the RASA process to a network interface."""
+    parser.add_argument(
+        "-i",
+        "--interface",
+        default=constants.DEFAULT_SERVER_INTERFACE,
+        type=str,
+        help="Network interface to run the server on.",
+    )
+
+
 # noinspection PyProtectedMember
 def add_port_argument(
     parser: Union[argparse.ArgumentParser, argparse._ArgumentGroup]
@@ -42,6 +55,27 @@ def add_server_arguments(parser: argparse.ArgumentParser) -> None:
         default=None,
         help="Store logs in specified file.",
     )
+    parser.add_argument(
+        "--use-syslog", action="store_true", help="Add syslog as a log handler"
+    )
+    parser.add_argument(
+        "--syslog-address",
+        type=str,
+        default=constants.DEFAULT_SYSLOG_HOST,
+        help="Address of the syslog server. --use-sylog flag is required",
+    )
+    parser.add_argument(
+        "--syslog-port",
+        type=int,
+        default=constants.DEFAULT_SYSLOG_PORT,
+        help="Port of the syslog server. --use-sylog flag is required",
+    )
+    parser.add_argument(
+        "--syslog-protocol",
+        type=str,
+        default=constants.DEFAULT_PROTOCOL,
+        help="Protocol used with the syslog server. Can be UDP (default) or TCP ",
+    )
     add_endpoint_param(
         parser,
         help_text="Configuration file for the model server and the connectors as a "
@@ -50,6 +84,8 @@ def add_server_arguments(parser: argparse.ArgumentParser) -> None:
 
     server_arguments = parser.add_argument_group("Server Settings")
 
+    add_interface_argument(server_arguments)
+
     add_port_argument(server_arguments)
 
     server_arguments.add_argument(
@@ -98,7 +134,6 @@ def add_server_arguments(parser: argparse.ArgumentParser) -> None:
         help="If your ssl-keyfile is protected by a password, you can specify it "
         "using this paramer.",
     )
-
     channel_arguments = parser.add_argument_group("Channels")
     channel_arguments.add_argument(
         "--credentials",
