@@ -1,4 +1,3 @@
-import os
 import json
 import logging
 from asyncio import AbstractEventLoop
@@ -9,7 +8,6 @@ from rasa.core.brokers.broker import EventBroker
 from rasa.shared.utils.io import DEFAULT_ENCODING
 from rasa.utils.endpoints import EndpointConfig
 from rasa.shared.exceptions import RasaException
-import rasa.shared.utils.common
 
 logger = logging.getLogger(__name__)
 
@@ -70,7 +68,6 @@ class KafkaEventBroker(EventBroker):
             security_protocol: Protocol used to communicate with brokers.
                 Valid values are: PLAINTEXT, SSL, SASL_PLAINTEXT, SASL_SSL.
             loglevel: Logging level of the kafka logger.
-
         """
         import kafka
 
@@ -145,7 +142,7 @@ class KafkaEventBroker(EventBroker):
 
         if self.security_protocol == "PLAINTEXT":
             authentication_params = dict(
-                security_protocol=self.security_protocol, ssl_check_hostname=False,
+                security_protocol=self.security_protocol, ssl_check_hostname=False
             )
         elif self.security_protocol == "SASL_PLAINTEXT":
             authentication_params = dict(
@@ -197,26 +194,10 @@ class KafkaEventBroker(EventBroker):
         else:
             partition_key = None
 
-        headers = []
-        if self.rasa_environment:
-            headers = [
-                (
-                    "RASA_ENVIRONMENT",
-                    bytes(self.rasa_environment, encoding=DEFAULT_ENCODING),
-                )
-            ]
-
         logger.debug(
-            f"Calling kafka send({self.topic}, value={event},"
-            f" key={partition_key!s}, headers={headers})"
+            f"Calling kafka send({self.topic}, value={event}, key={partition_key!s})"
         )
-
-        self.producer.send(self.topic, value=event, key=partition_key, headers=headers)
+        self.producer.send(self.topic, value=event, key=partition_key)
 
     def _close(self) -> None:
         self.producer.close()
-
-    @rasa.shared.utils.common.lazy_property
-    def rasa_environment(self) -> Optional[Text]:
-        """Get value of the `RASA_ENVIRONMENT` environment variable."""
-        return os.environ.get("RASA_ENVIRONMENT")
