@@ -314,7 +314,6 @@ class SlackInput(InputChannel):
     @staticmethod
     def _get_interactive_response(action: Dict) -> Optional[Text]:
         """Parse the payload for the response value."""
-
         if action["type"] == "button":
             return action.get("value")
         elif action["type"] == "select":
@@ -334,6 +333,8 @@ class SlackInput(InputChannel):
         elif action["type"] == "datepicker":
             return action.get("selected_date")
 
+        return None
+
     async def process_message(
         self,
         request: Request,
@@ -355,7 +356,7 @@ class SlackInput(InputChannel):
             )
 
             return response.text(
-                None, status=HTTPStatus.CREATED, headers={"X-Slack-No-Retry": 1}  # type: ignore[dict-item]  # noqa: E501
+                None, status=HTTPStatus.CREATED, headers={"X-Slack-No-Retry": "1"}
             )
 
         if metadata is not None:
