@@ -224,7 +224,7 @@ class TwilioVoiceInput(InputChannel):
             return response.json({"status": "ok"})
 
         @twilio_voice_webhook.route("/webhook", methods=["POST"])
-        async def receive(request: Request) -> Text:
+        async def receive(request: Request) -> HTTPResponse:
             sender_id = request.form.get("From")
             text = request.form.get("SpeechResult")
             input_channel = self.name()
@@ -239,16 +239,14 @@ class TwilioVoiceInput(InputChannel):
             # determine the response.
             if text is not None:
                 await on_new_message(
-                    UserMessage(
-                        text, collector, sender_id, input_channel=input_channel,
-                    )
+                    UserMessage(text, collector, sender_id, input_channel=input_channel)
                 )
 
                 twilio_response = self._build_twilio_voice_response(collector.messages)
             # If the user doesn't respond resend the last message.
             else:
                 # Get last user utterance from tracker.
-                tracker = request.app.ctx.agent.tracker_store.retrieve(sender_id)
+                tracker = request.app.agent.tracker_store.retrieve(sender_id)
                 last_response = None
                 if tracker:
                     last_response = next(
@@ -313,7 +311,7 @@ class TwilioVoiceCollectingOutputChannel(CollectingOutputChannel):
         return "twilio_voice"
 
     @staticmethod
-    def _emoji_warning(text: Text,) -> None:
+    def _emoji_warning(text: Text) -> None:
         """Raises a warning if text contains an emoji."""
         emoji_regex = rasa.utils.io.get_emoji_regex()
         if emoji_regex.findall(text):
