@@ -2,7 +2,6 @@ import logging
 from typing import Optional, Union, Text, Any, Dict
 
 import rasa.shared.utils.common
-from rasa.shared.constants import DOCS_URL_MIGRATION_GUIDE
 from rasa.shared.core.domain import Domain
 from rasa.utils.endpoints import EndpointConfig
 from rasa.shared.core.trackers import DialogueStateTracker
@@ -61,14 +60,6 @@ def _create_from_endpoint_config(
     elif endpoint_config.type.lower() == "response":
         from rasa.core.nlg import TemplatedNaturalLanguageGenerator
 
-        nlg = TemplatedNaturalLanguageGenerator(domain.responses)
-    elif endpoint_config.type.lower() == "template":
-        rasa.shared.utils.io.raise_deprecation_warning(
-            "Please change the NLG endpoint config type from `template` to `response`.",
-            docs=f"{DOCS_URL_MIGRATION_GUIDE}#rasa-23-to-rasa-24",
-        )
-        from rasa.core.nlg import TemplatedNaturalLanguageGenerator
-
         nlg = TemplatedNaturalLanguageGenerator(domain.responses)
     else:
         nlg = _load_from_module_name_in_endpoint_config(endpoint_config, domain)
