@@ -3,8 +3,6 @@ import itertools
 import json
 import logging
 import os
-import pickle
-from datetime import datetime, timezone
 
 from time import sleep
 from typing import (
@@ -43,10 +41,11 @@ from rasa.shared.core.trackers import (
     DialogueStateTracker,
     EventVerbosity,
 )
-from rasa.shared.exceptions import ConnectionException
+from rasa.shared.exceptions import ConnectionException, RasaException
 from rasa.shared.nlu.constants import INTENT_NAME_KEY
 from rasa.utils.endpoints import EndpointConfig
 import sqlalchemy as sa
+from sqlalchemy.ext.declarative import declarative_base, DeclarativeMeta
 
 if TYPE_CHECKING:
     import boto3.resources.factory.dynamodb.Table
@@ -65,6 +64,10 @@ POSTGRESQL_DEFAULT_POOL_SIZE = 50
 DEFAULT_REDIS_TRACKER_STORE_KEY_PREFIX = "tracker:"
 
 
+class TrackerDeserialisationException(RasaException):
+    """Raised when an error is encountered while deserialising a tracker."""
+
+
 class TrackerStore:
     """Represents common behavior and interface for all `TrackerStore`s."""
 
@@ -86,26 +89,6 @@ class TrackerStore:
         self.event_broker = event_broker
         self.max_event_history = None
 
-        # TODO: Remove this in Rasa Open Source 3.0
-        self.retrieve_events_from_previous_conversation_sessions: Optional[bool] = None
-        self._set_deprecated_kwargs_and_emit_warning(kwargs)
-
-    def _set_deprecated_kwargs_and_emit_warning(self, kwargs: Dict[Text, Any]) -> None:
-        retrieve_events_from_previous_conversation_sessions = kwargs.get(
-            "retrieve_events_from_previous_conversation_sessions"
-        )
-
-        if retrieve_events_from_previous_conversation_sessions is not None:
-            rasa.shared.utils.io.raise_deprecation_warning(
-                f"Specifying the `retrieve_events_from_previous_conversation_sessions` "
-                f"kwarg for the `{self.__class__.__name__}` class is deprecated and "
-                f"will be removed in Rasa Open Source 3.0. "
-                f"Please use the `retrieve_full_tracker()` method instead."
-            )
-            self.retrieve_events_from_previous_conversation_sessions = (
-                retrieve_events_from_previous_conversation_sessions
-            )
-
     @staticmethod
     def create(
         obj: Union["TrackerStore", EndpointConfig, None],
@@ -261,34 +244,20 @@ class TrackerStore:
 
         return json.dumps(dialogue.as_dict())
 
-    @staticmethod
-    def _deserialize_dialogue_from_pickle(
-        sender_id: Text, serialised_tracker: bytes
-    ) -> Dialogue:
-        # TODO: Remove in Rasa Open Source 3.0
-        rasa.shared.utils.io.raise_deprecation_warning(
-            f"Found pickled tracker for "
-            f"conversation ID '{sender_id}'. Deserialization of pickled "
-            f"trackers is deprecated and will be removed in Rasa Open Source 3.0. Rasa "
-            f"will perform any future save operations of this tracker using json "
-            f"serialisation."
-        )
-
-        return pickle.loads(serialised_tracker)
-
     def deserialise_tracker(
         self, sender_id: Text, serialised_tracker: Union[Text, bytes]
     ) -> Optional[DialogueStateTracker]:
         """Deserializes the tracker and returns it."""
-
         tracker = self.init_tracker(sender_id)
 
         try:
             dialogue = Dialogue.from_parameters(json.loads(serialised_tracker))
-        except UnicodeDecodeError:
-            dialogue = self._deserialize_dialogue_from_pickle(
-                sender_id, serialised_tracker
-            )
+        except UnicodeDecodeError as e:
+            raise TrackerDeserialisationException(
+                "Tracker cannot be deserialised. "
+                "Trackers must be serialised as json. "
+                "Support for deserialising pickled trackers has been removed."
+            ) from e
 
         tracker.recreate_from_dialogue(dialogue)
 
@@ -449,9 +418,9 @@ class DynamoTrackerStore(TrackerStore):
         except self.client.exceptions.ResourceNotFoundException:
             table = dynamo.create_table(
                 TableName=self.table_name,
-                KeySchema=[{"AttributeName": "sender_id", "KeyType": "HASH"},],
+                KeySchema=[{"AttributeName": "sender_id", "KeyType": "HASH"}],
                 AttributeDefinitions=[
-                    {"AttributeName": "sender_id", "AttributeType": "S"},
+                    {"AttributeName": "sender_id", "AttributeType": "S"}
                 ],
                 ProvisionedThroughput={"ReadCapacityUnits": 5, "WriteCapacityUnits": 5},
             )
@@ -461,66 +430,28 @@ class DynamoTrackerStore(TrackerStore):
         else:
             table = dynamo.Table(table_name)
 
-            column_names = [
-                attribute["AttributeName"] for attribute in table.attribute_definitions
-            ]
-            if "session_date" in column_names:
-                rasa.shared.utils.io.raise_warning(
-                    "Attribute 'session_date' is no longer required when using a "
-                    "DynamoDB TrackerStore. Please remove this attribute from "
-                    "any existing tables.",
-                    FutureWarning,
-                )
-
         return table
 
     def save(self, tracker: DialogueStateTracker) -> None:
         """Saves the current conversation state."""
-        from botocore.exceptions import ClientError
-
         if self.event_broker:
             self.stream_events(tracker)
         serialized = self.serialise_tracker(tracker)
 
-        try:
-            self.db.put_item(Item=serialized)
-        except ClientError as e:
-            if "Missing the key session_date" in repr(e):
-                # the session_date attribute got removed as it was useless
-                # old databases will still contain an attribute for it though
-                # which we need to set (otherwise we are getting the error we
-                # just ran into) this section should be removed in 3.0
-                legacy_date = self._retrieve_latest_session_date(tracker.sender_id)
-
-                serialized["session_date"] = legacy_date
-                self.db.put_item(Item=serialized)
-            else:
-                raise
-
-    def _retrieve_latest_session_date(self, sender_id: Text) -> Optional[int]:
-        dialogues = self.db.query(
-            KeyConditionExpression=Key("sender_id").eq(sender_id),
-            Limit=1,
-            ScanIndexForward=False,
-        )["Items"]
-
-        if not dialogues:
-            return int(datetime.now(tz=timezone.utc).timestamp())
-
-        return dialogues[0].get("session_date")
+        self.db.put_item(Item=serialized)
 
     def serialise_tracker(self, tracker: "DialogueStateTracker") -> Dict:
         """Serializes the tracker, returns object with decimal types."""
         d = tracker.as_dialogue().as_dict()
-        d.update(
-            {"sender_id": tracker.sender_id,}
-        )
+        d.update({"sender_id": tracker.sender_id})
         # DynamoDB cannot store `float`s, so we'll convert them to `Decimal`s
         return core_utils.replace_floats_with_decimals(d)
 
     def retrieve(self, sender_id: Text) -> Optional[DialogueStateTracker]:
-        # Retrieve dialogues for a sender_id in reverse-chronological order based on
-        # the session_date sort key
+        """Retrieve dialogues for a sender_id in reverse-chronological order.
+
+        Based on the session_date sort key.
+        """
         dialogues = self.db.query(
             KeyConditionExpression=Key("sender_id").eq(sender_id),
             Limit=1,
@@ -541,10 +472,17 @@ class DynamoTrackerStore(TrackerStore):
 
     def keys(self) -> Iterable[Text]:
         """Returns sender_ids of the `DynamoTrackerStore`."""
-        return [
-            i["sender_id"]
-            for i in self.db.scan(ProjectionExpression="sender_id")["Items"]
-        ]
+        response = self.db.scan(ProjectionExpression="sender_id")
+        sender_ids = [i["sender_id"] for i in response["Items"]]
+
+        while response.get("LastEvaluatedKey"):
+            response = self.db.scan(
+                ProjectionExpression="sender_id",
+                ExclusiveStartKey=response["LastEvaluatedKey"],
+            )
+            sender_ids.extend([i["sender_id"] for i in response["Items"]])
+
+        return sender_ids
 
 
 class MongoTrackerStore(TrackerStore):
@@ -633,12 +571,10 @@ class MongoTrackerStore(TrackerStore):
 
         stored = self.conversations.find_one({"sender_id": tracker.sender_id}) or {}
         all_events = self._events_from_serialized_tracker(stored)
-        if self.retrieve_events_from_previous_conversation_sessions:
-            number_events_since_last_session = len(all_events)
-        else:
-            number_events_since_last_session = len(
-                self._events_since_last_session_start(all_events)
-            )
+
+        number_events_since_last_session = len(
+            self._events_since_last_session_start(all_events)
+        )
 
         return itertools.islice(
             tracker.events, number_events_since_last_session, len(tracker.events)
@@ -696,11 +632,7 @@ class MongoTrackerStore(TrackerStore):
         return events
 
     def retrieve(self, sender_id: Text) -> Optional[DialogueStateTracker]:
-        # TODO: Remove this in Rasa Open Source 3.0 along with the
-        # deprecation warning in the constructor
-        if self.retrieve_events_from_previous_conversation_sessions:
-            return self.retrieve_full_tracker(sender_id)
-
+        """Retrieves tracker for the latest conversation session."""
         events = self._retrieve(sender_id, fetch_events_from_all_sessions=False)
 
         if not events:
@@ -711,6 +643,7 @@ class MongoTrackerStore(TrackerStore):
     def retrieve_full_tracker(
         self, conversation_id: Text
     ) -> Optional[DialogueStateTracker]:
+        """Fetching all tracker events across conversation sessions."""
         events = self._retrieve(conversation_id, fetch_events_from_all_sessions=True)
 
         if not events:
@@ -818,11 +751,24 @@ def ensure_schema_exists(session: "Session") -> None:
             raise ValueError(schema_name)
 
 
+def validate_port(port: Any) -> Optional[int]:
+    """Ensure that port can be converted to integer.
+
+    Raises:
+        RasaException if port cannot be cast to integer.
+    """
+    if port is not None and not isinstance(port, int):
+        try:
+            port = int(port)
+        except ValueError as e:
+            raise RasaException(f"The port '{port}' cannot be cast to integer.") from e
+
+    return port
+
+
 class SQLTrackerStore(TrackerStore):
     """Store which can save and retrieve trackers from an SQL database."""
 
-    from sqlalchemy.ext.declarative import declarative_base, DeclarativeMeta
-
     Base: DeclarativeMeta = declarative_base()
 
     class SQLEvent(Base):
@@ -856,6 +802,8 @@ class SQLTrackerStore(TrackerStore):
     ) -> None:
         import sqlalchemy.exc
 
+        port = validate_port(port)
+
         engine_url = self.get_db_url(
             dialect, host, port, db, username, password, login_db, query
         )
@@ -959,7 +907,7 @@ class SQLTrackerStore(TrackerStore):
 
         if not self.engine.dialect.name == "postgresql":
             rasa.shared.utils.io.raise_warning(
-                "The parameter 'login_db' can only be used with a postgres database.",
+                "The parameter 'login_db' can only be used with a postgres database."
             )
             return
 
@@ -1024,22 +972,19 @@ class SQLTrackerStore(TrackerStore):
             session.close()
 
     def keys(self) -> Iterable[Text]:
-        """Returns sender_ids of the SQLTrackerStore"""
+        """Returns sender_ids of the SQLTrackerStore."""
         with self.session_scope() as session:
             sender_ids = session.query(self.SQLEvent.sender_id).distinct().all()
             return [sender_id for (sender_id,) in sender_ids]
 
     def retrieve(self, sender_id: Text) -> Optional[DialogueStateTracker]:
-        # TODO: Remove this in Rasa Open Source 3.0 along with the
-        # deprecation warning in the constructor
-        if self.retrieve_events_from_previous_conversation_sessions:
-            return self.retrieve_full_tracker(sender_id)
-
+        """Retrieves tracker for the latest conversation session."""
         return self._retrieve(sender_id, fetch_events_from_all_sessions=False)
 
     def retrieve_full_tracker(
         self, conversation_id: Text
     ) -> Optional[DialogueStateTracker]:
+        """Fetching all tracker events across conversation sessions."""
         return self._retrieve(conversation_id, fetch_events_from_all_sessions=True)
 
     def _retrieve(
@@ -1146,11 +1091,7 @@ class SQLTrackerStore(TrackerStore):
     ) -> Iterator:
         """Return events from the tracker which aren't currently stored."""
         number_of_events_since_last_session = self._event_query(
-            session,
-            tracker.sender_id,
-            fetch_events_from_all_sessions=(
-                self.retrieve_events_from_previous_conversation_sessions
-            ),
+            session, tracker.sender_id, fetch_events_from_all_sessions=False
         ).count()
 
         return itertools.islice(
