@@ -6,10 +6,7 @@ from dataclasses import dataclass, field
 import logging
 from typing import Any, Callable, Dict, List, Optional, Text, Type, Tuple
 
-from rasa.engine.exceptions import (
-    GraphComponentException,
-    GraphSchemaException,
-)
+from rasa.engine.exceptions import GraphComponentException, GraphSchemaException
 import rasa.shared.utils.common
 import rasa.utils.common
 from rasa.engine.storage.resource import Resource
@@ -361,7 +358,7 @@ class GraphNode:
             self._component_class, self._constructor_name
         )
         self._component_config: Dict[Text, Any] = rasa.utils.common.override_defaults(
-            self._component_class.get_default_config(), component_config,
+            self._component_class.get_default_config(), component_config
         )
         self._fn_name: Text = fn_name
         self._fn: Callable = getattr(self._component_class, self._fn_name)
