@@ -3,21 +3,24 @@ import os
 from typing import Text, Dict, Optional, List, Any, Iterable, Tuple, Union
 from pathlib import Path
 
+from rasa.core.agent import Agent
+from rasa.engine.storage.local_model_storage import LocalModelStorage
 import rasa.shared.utils.cli
 import rasa.shared.utils.common
 import rasa.shared.utils.io
 import rasa.utils.common
 from rasa.constants import RESULTS_FILE, NUMBER_OF_TRAINING_STORIES_FILE
-from rasa.shared.constants import DEFAULT_RESULTS_PATH
 from rasa.exceptions import ModelNotFound
+from rasa.shared.constants import DEFAULT_RESULTS_PATH
 import rasa.shared.nlu.training_data.loading
+from rasa.shared.importers.autoconfig import TrainingType
 from rasa.shared.nlu.training_data.training_data import TrainingData
-
+import rasa.model
 
 logger = logging.getLogger(__name__)
 
 
-def test_core_models_in_directory(
+async def test_core_models_in_directory(
     model_directory: Text,
     stories: Text,
     output: Text,
@@ -36,13 +39,11 @@ def test_core_models_in_directory(
 
     model_directory = _get_sanitized_model_directory(model_directory)
 
-    rasa.utils.common.run_in_loop(
-        compare_models_in_dir(
-            model_directory,
-            stories,
-            output,
-            use_conversation_test_files=use_conversation_test_files,
-        )
+    await compare_models_in_dir(
+        model_directory,
+        stories,
+        output,
+        use_conversation_test_files=use_conversation_test_files,
     )
 
     story_n_path = os.path.join(model_directory, NUMBER_OF_TRAINING_STORIES_FILE)
@@ -85,7 +86,6 @@ def _get_sanitized_model_directory(model_directory: Text) -> Text:
     Returns: The adjusted model_directory that should be used in
         `test_core_models_in_directory`.
     """
-    import rasa.model
 
     p = Path(model_directory)
     if p.is_file():
@@ -99,7 +99,7 @@ def _get_sanitized_model_directory(model_directory: Text) -> Text:
     return model_directory
 
 
-def test_core_models(
+async def test_core_models(
     models: List[Text],
     stories: Text,
     output: Text,
@@ -116,21 +116,12 @@ def test_core_models(
     """
     from rasa.core.test import compare_models
 
-    rasa.utils.common.run_in_loop(
-        compare_models(
-            models,
-            stories,
-            output,
-            use_conversation_test_files=use_conversation_test_files,
-        )
+    await compare_models(
+        models, stories, output, use_conversation_test_files=use_conversation_test_files
     )
 
 
-# backwards compatibility
-test = rasa.test
-
-
-def test_core(
+async def test_core(
     model: Optional[Text] = None,
     stories: Optional[Text] = None,
     output: Text = DEFAULT_RESULTS_PATH,
@@ -138,18 +129,8 @@ def test_core(
     use_conversation_test_files: bool = False,
 ) -> None:
     """Tests a trained Core model against a set of test stories."""
-    import rasa.model
-    from rasa.shared.nlu.interpreter import RegexInterpreter
-    from rasa.core.agent import Agent
-
-    if additional_arguments is None:
-        additional_arguments = {}
-
-    if output:
-        rasa.shared.utils.io.create_directory(output)
-
     try:
-        unpacked_model = rasa.model.get_model(model)
+        model = rasa.model.get_local_model(model)
     except ModelNotFound:
         rasa.shared.utils.cli.print_error(
             "Unable to test: could not find a model. Use 'rasa train' to train a "
@@ -157,36 +138,43 @@ def test_core(
         )
         return
 
-    _agent = Agent.load(unpacked_model)
-
-    if _agent.policy_ensemble is None:
+    metadata = LocalModelStorage.metadata_from_archive(model)
+    if metadata.training_type == TrainingType.NLU:
         rasa.shared.utils.cli.print_error(
-            "Unable to test: could not find a Core model. Use 'rasa train' to train a "
+            "Unable to test: no core model found. Use 'rasa train' to train a "
             "Rasa model and provide it via the '--model' argument."
         )
-
-    if isinstance(_agent.interpreter, RegexInterpreter):
+    elif metadata.training_type == TrainingType.CORE and use_conversation_test_files:
         rasa.shared.utils.cli.print_warning(
-            "No NLU model found. Using default 'RegexInterpreter' for end-to-end "
+            "No NLU model found. Using default 'RegexMessageHandler' for end-to-end "
             "evaluation. If you added actual user messages to your test stories "
             "this will likely lead to the tests failing. In that case, you need "
             "to train a NLU model first, e.g. using `rasa train`."
         )
 
+    if additional_arguments is None:
+        additional_arguments = {}
+
+    if output:
+        rasa.shared.utils.io.create_directory(output)
+
+    _agent = Agent.load(model_path=model)
+
+    if not _agent.is_ready():
+        rasa.shared.utils.cli.print_error(
+            "Unable to test: processor not loaded. Use 'rasa train' to train a "
+            "Rasa model and provide it via the '--model' argument."
+        )
+        return
+
     from rasa.core.test import test as core_test
 
     kwargs = rasa.shared.utils.common.minimal_kwargs(
         additional_arguments, core_test, ["stories", "agent", "e2e"]
     )
 
-    rasa.utils.common.run_in_loop(
-        core_test(
-            stories,
-            _agent,
-            e2e=use_conversation_test_files,
-            out_directory=output,
-            **kwargs,
-        )
+    await core_test(
+        stories, _agent, e2e=use_conversation_test_files, out_directory=output, **kwargs
     )
 
 
@@ -198,10 +186,11 @@ async def test_nlu(
 ) -> None:
     """Tests the NLU Model."""
     from rasa.nlu.test import run_evaluation
-    from rasa.model import get_model
+
+    rasa.shared.utils.io.create_directory(output_directory)
 
     try:
-        unpacked_model = get_model(model)
+        model = rasa.model.get_local_model(model)
     except ModelNotFound:
         rasa.shared.utils.cli.print_error(
             "Could not find any model. Use 'rasa train nlu' to train a "
@@ -209,16 +198,15 @@ async def test_nlu(
         )
         return
 
-    rasa.shared.utils.io.create_directory(output_directory)
+    metadata = LocalModelStorage.metadata_from_archive(model)
 
-    nlu_model = os.path.join(unpacked_model, "nlu")
-
-    if os.path.exists(nlu_model):
+    if os.path.exists(model) and metadata.training_type != TrainingType.CORE:
         kwargs = rasa.shared.utils.common.minimal_kwargs(
             additional_arguments, run_evaluation, ["data_path", "model"]
         )
+        _agent = Agent.load(model_path=model)
         await run_evaluation(
-            nlu_data, nlu_model, output_directory=output_directory, **kwargs
+            nlu_data, _agent.processor, output_directory=output_directory, **kwargs
         )
     else:
         rasa.shared.utils.cli.print_error(
@@ -288,8 +276,8 @@ def plot_nlu_results(output_directory: Text, number_of_examples: List[int]) -> N
     )
 
 
-def perform_nlu_cross_validation(
-    config: Text,
+async def perform_nlu_cross_validation(
+    config: Dict[Text, Any],
     data: TrainingData,
     output: Text,
     additional_arguments: Optional[Dict[Text, Any]],
@@ -303,7 +291,6 @@ def perform_nlu_cross_validation(
         additional_arguments: Additional arguments which are passed to the
             cross-validation, like number of `disable_plotting`.
     """
-    import rasa.nlu.config
     from rasa.nlu.test import (
         drop_intents_below_freq,
         cross_validate,
@@ -313,13 +300,14 @@ def perform_nlu_cross_validation(
 
     additional_arguments = additional_arguments or {}
     folds = int(additional_arguments.get("folds", 3))
-    nlu_config = rasa.nlu.config.load(config)
+
     data = drop_intents_below_freq(data, cutoff=folds)
     kwargs = rasa.shared.utils.common.minimal_kwargs(
         additional_arguments, cross_validate
     )
-    results, entity_results, response_selection_results = cross_validate(
-        data, folds, nlu_config, output, **kwargs
+
+    results, entity_results, response_selection_results = await cross_validate(
+        data, folds, config, output, **kwargs
     )
     logger.info(f"CV evaluation (n={folds})")
 
