@@ -1,27 +1,30 @@
+from __future__ import annotations
 import copy
 import logging
+from rasa.nlu.featurizers.featurizer import Featurizer
 
 import numpy as np
 import tensorflow as tf
 
 from typing import Any, Dict, Optional, Text, Tuple, Union, List, Type
 
+from rasa.engine.graph import ExecutionContext
+from rasa.engine.recipes.default_recipe import DefaultV1Recipe
+from rasa.engine.storage.resource import Resource
+from rasa.engine.storage.storage import ModelStorage
 from rasa.shared.constants import DIAGNOSTIC_DATA
 from rasa.shared.nlu.training_data import util
 import rasa.shared.utils.io
 from rasa.shared.exceptions import InvalidConfigException
 from rasa.shared.nlu.training_data.training_data import TrainingData
 from rasa.shared.nlu.training_data.message import Message
-from rasa.nlu.components import Component
-from rasa.nlu.featurizers.featurizer import Featurizer
-from rasa.nlu.model import Metadata
 from rasa.nlu.classifiers.diet_classifier import (
-    DIETClassifier,
     DIET,
     LABEL_KEY,
     LABEL_SUB_KEY,
     SENTENCE,
     SEQUENCE,
+    DIETClassifier,
 )
 from rasa.nlu.extractors.extractor import EntityTagSpec
 from rasa.utils.tensorflow import rasa_layers
@@ -38,6 +41,7 @@ from rasa.utils.tensorflow.constants import (
     RANDOM_SEED,
     LEARNING_RATE,
     RANKING_LENGTH,
+    RENORMALIZE_CONFIDENCES,
     LOSS_TYPE,
     SIMILARITY_TYPE,
     NUM_NEG,
@@ -82,12 +86,11 @@ from rasa.nlu.constants import (
     RESPONSE_SELECTOR_PROPERTY_NAME,
     RESPONSE_SELECTOR_RETRIEVAL_INTENTS,
     RESPONSE_SELECTOR_RESPONSES_KEY,
-    RESPONSE_SELECTOR_RESPONSE_TEMPLATES_KEY,
     RESPONSE_SELECTOR_PREDICTION_KEY,
     RESPONSE_SELECTOR_RANKING_KEY,
     RESPONSE_SELECTOR_UTTER_ACTION_KEY,
-    RESPONSE_SELECTOR_TEMPLATE_NAME_KEY,
     RESPONSE_SELECTOR_DEFAULT_INTENT,
+    DEFAULT_TRANSFORMER_SIZE,
 )
 from rasa.shared.nlu.constants import (
     TEXT,
@@ -104,6 +107,9 @@ from rasa.utils.tensorflow.models import RasaModel
 logger = logging.getLogger(__name__)
 
 
+@DefaultV1Recipe.register(
+    DefaultV1Recipe.ComponentType.INTENT_CLASSIFIER, is_trainable=True
+)
 class ResponseSelector(DIETClassifier):
     """Response selector using supervised embeddings.
 
@@ -124,146 +130,163 @@ class ResponseSelector(DIETClassifier):
     """
 
     @classmethod
-    def required_components(cls) -> List[Type[Component]]:
+    def required_components(cls) -> List[Type]:
+        """Components that should be included in the pipeline before this component."""
         return [Featurizer]
 
-    defaults = {
-        # ## Architecture of the used neural network
-        # Hidden layer sizes for layers before the embedding layers for user message
-        # and labels.
-        # The number of hidden layers is equal to the length of the corresponding
-        # list.
-        HIDDEN_LAYERS_SIZES: {TEXT: [256, 128], LABEL: [256, 128]},
-        # Whether to share the hidden layer weights between input words and responses
-        SHARE_HIDDEN_LAYERS: False,
-        # Number of units in transformer
-        TRANSFORMER_SIZE: None,
-        # Number of transformer layers
-        NUM_TRANSFORMER_LAYERS: 0,
-        # Number of attention heads in transformer
-        NUM_HEADS: 4,
-        # If 'True' use key relative embeddings in attention
-        KEY_RELATIVE_ATTENTION: False,
-        # If 'True' use key relative embeddings in attention
-        VALUE_RELATIVE_ATTENTION: False,
-        # Max position for relative embeddings
-        MAX_RELATIVE_POSITION: None,
-        # Use a unidirectional or bidirectional encoder.
-        UNIDIRECTIONAL_ENCODER: False,
-        # ## Training parameters
-        # Initial and final batch sizes:
-        # Batch size will be linearly increased for each epoch.
-        BATCH_SIZES: [64, 256],
-        # Strategy used when creating batches.
-        # Can be either 'sequence' or 'balanced'.
-        BATCH_STRATEGY: BALANCED,
-        # Number of epochs to train
-        EPOCHS: 300,
-        # Set random seed to any 'int' to get reproducible results
-        RANDOM_SEED: None,
-        # Initial learning rate for the optimizer
-        LEARNING_RATE: 0.001,
-        # ## Parameters for embeddings
-        # Dimension size of embedding vectors
-        EMBEDDING_DIMENSION: 20,
-        # Default dense dimension to use if no dense features are present.
-        DENSE_DIMENSION: {TEXT: 512, LABEL: 512},
-        # Default dimension to use for concatenating sequence and sentence features.
-        CONCAT_DIMENSION: {TEXT: 512, LABEL: 512},
-        # The number of incorrect labels. The algorithm will minimize
-        # their similarity to the user input during training.
-        NUM_NEG: 20,
-        # Type of similarity measure to use, either 'auto' or 'cosine' or 'inner'.
-        SIMILARITY_TYPE: AUTO,
-        # The type of the loss function, either 'cross_entropy' or 'margin'.
-        LOSS_TYPE: CROSS_ENTROPY,
-        # Number of top actions to normalize scores for. Applicable with
-        # loss type 'cross_entropy' and 'softmax' confidences. Set to 0
-        # to turn off normalization.
-        RANKING_LENGTH: 10,
-        # Indicates how similar the algorithm should try to make embedding vectors
-        # for correct labels.
-        # Should be 0.0 < ... < 1.0 for 'cosine' similarity type.
-        MAX_POS_SIM: 0.8,
-        # Maximum negative similarity for incorrect labels.
-        # Should be -1.0 < ... < 1.0 for 'cosine' similarity type.
-        MAX_NEG_SIM: -0.4,
-        # If 'True' the algorithm only minimizes maximum similarity over
-        # incorrect intent labels, used only if 'loss_type' is set to 'margin'.
-        USE_MAX_NEG_SIM: True,
-        # Scale loss inverse proportionally to confidence of correct prediction
-        SCALE_LOSS: True,
-        # ## Regularization parameters
-        # The scale of regularization
-        REGULARIZATION_CONSTANT: 0.002,
-        # Fraction of trainable weights in internal layers.
-        CONNECTION_DENSITY: 1.0,
-        # The scale of how important is to minimize the maximum similarity
-        # between embeddings of different labels.
-        NEGATIVE_MARGIN_SCALE: 0.8,
-        # Dropout rate for encoder
-        DROP_RATE: 0.2,
-        # Dropout rate for attention
-        DROP_RATE_ATTENTION: 0,
-        # If 'True' apply dropout to sparse input tensors
-        SPARSE_INPUT_DROPOUT: False,
-        # If 'True' apply dropout to dense input tensors
-        DENSE_INPUT_DROPOUT: False,
-        # ## Evaluation parameters
-        # How often calculate validation accuracy.
-        # Small values may hurt performance, e.g. model accuracy.
-        EVAL_NUM_EPOCHS: 20,
-        # How many examples to use for hold out validation set
-        # Large values may hurt performance, e.g. model accuracy.
-        EVAL_NUM_EXAMPLES: 0,
-        # ## Selector config
-        # If 'True' random tokens of the input message will be masked and the model
-        # should predict those tokens.
-        MASKED_LM: False,
-        # Name of the intent for which this response selector is to be trained
-        RETRIEVAL_INTENT: None,
-        # Boolean flag to check if actual text of the response
-        # should be used as ground truth label for training the model.
-        USE_TEXT_AS_LABEL: False,
-        # If you want to use tensorboard to visualize training and validation metrics,
-        # set this option to a valid output directory.
-        TENSORBOARD_LOG_DIR: None,
-        # Define when training metrics for tensorboard should be logged.
-        # Either after every epoch or for every training step.
-        # Valid values: 'epoch' and 'batch'
-        TENSORBOARD_LOG_LEVEL: "epoch",
-        # Specify what features to use as sequence and sentence features
-        # By default all features in the pipeline are used.
-        FEATURIZERS: [],
-        # Perform model checkpointing
-        CHECKPOINT_MODEL: False,
-        # if 'True' applies sigmoid on all similarity terms and adds it
-        # to the loss function to ensure that similarity values are
-        # approximately bounded. Used inside softmax loss only.
-        CONSTRAIN_SIMILARITIES: False,
-        # Model confidence to be returned during inference. Possible values -
-        # 'softmax' and 'linear_norm'.
-        MODEL_CONFIDENCE: SOFTMAX,
-    }
-
-    # The `transformer_size` to use as a default when the transformer is enabled.
-    default_transformer_size_when_enabled = 256
+    @staticmethod
+    def get_default_config() -> Dict[Text, Any]:
+        """The component's default config (see parent class for full docstring)."""
+        return {
+            **DIETClassifier.get_default_config(),
+            # ## Architecture of the used neural network
+            # Hidden layer sizes for layers before the embedding layers for user message
+            # and labels.
+            # The number of hidden layers is equal to the length of the corresponding
+            # list.
+            HIDDEN_LAYERS_SIZES: {TEXT: [256, 128], LABEL: [256, 128]},
+            # Whether to share the hidden layer weights between input words
+            # and responses
+            SHARE_HIDDEN_LAYERS: False,
+            # Number of units in transformer
+            TRANSFORMER_SIZE: None,
+            # Number of transformer layers
+            NUM_TRANSFORMER_LAYERS: 0,
+            # Number of attention heads in transformer
+            NUM_HEADS: 4,
+            # If 'True' use key relative embeddings in attention
+            KEY_RELATIVE_ATTENTION: False,
+            # If 'True' use key relative embeddings in attention
+            VALUE_RELATIVE_ATTENTION: False,
+            # Max position for relative embeddings. Only in effect if key-
+            # or value relative attention are turned on
+            MAX_RELATIVE_POSITION: 5,
+            # Use a unidirectional or bidirectional encoder.
+            UNIDIRECTIONAL_ENCODER: False,
+            # ## Training parameters
+            # Initial and final batch sizes:
+            # Batch size will be linearly increased for each epoch.
+            BATCH_SIZES: [64, 256],
+            # Strategy used when creating batches.
+            # Can be either 'sequence' or 'balanced'.
+            BATCH_STRATEGY: BALANCED,
+            # Number of epochs to train
+            EPOCHS: 300,
+            # Set random seed to any 'int' to get reproducible results
+            RANDOM_SEED: None,
+            # Initial learning rate for the optimizer
+            LEARNING_RATE: 0.001,
+            # ## Parameters for embeddings
+            # Dimension size of embedding vectors
+            EMBEDDING_DIMENSION: 20,
+            # Default dense dimension to use if no dense features are present.
+            DENSE_DIMENSION: {TEXT: 512, LABEL: 512},
+            # Default dimension to use for concatenating sequence and sentence features.
+            CONCAT_DIMENSION: {TEXT: 512, LABEL: 512},
+            # The number of incorrect labels. The algorithm will minimize
+            # their similarity to the user input during training.
+            NUM_NEG: 20,
+            # Type of similarity measure to use, either 'auto' or 'cosine' or 'inner'.
+            SIMILARITY_TYPE: AUTO,
+            # The type of the loss function, either 'cross_entropy' or 'margin'.
+            LOSS_TYPE: CROSS_ENTROPY,
+            # Number of top actions for which confidences should be predicted.
+            # Set to 0 if confidences for all intents should be reported.
+            RANKING_LENGTH: 10,
+            # Determines whether the confidences of the chosen top actions should be
+            # renormalized so that they sum up to 1. By default, we do not renormalize
+            # and return the confidences for the top actions as is.
+            # Note that renormalization only makes sense if confidences are generated
+            # via `softmax`.
+            RENORMALIZE_CONFIDENCES: False,
+            # Indicates how similar the algorithm should try to make embedding vectors
+            # for correct labels.
+            # Should be 0.0 < ... < 1.0 for 'cosine' similarity type.
+            MAX_POS_SIM: 0.8,
+            # Maximum negative similarity for incorrect labels.
+            # Should be -1.0 < ... < 1.0 for 'cosine' similarity type.
+            MAX_NEG_SIM: -0.4,
+            # If 'True' the algorithm only minimizes maximum similarity over
+            # incorrect intent labels, used only if 'loss_type' is set to 'margin'.
+            USE_MAX_NEG_SIM: True,
+            # Scale loss inverse proportionally to confidence of correct prediction
+            SCALE_LOSS: True,
+            # ## Regularization parameters
+            # The scale of regularization
+            REGULARIZATION_CONSTANT: 0.002,
+            # Fraction of trainable weights in internal layers.
+            CONNECTION_DENSITY: 1.0,
+            # The scale of how important is to minimize the maximum similarity
+            # between embeddings of different labels.
+            NEGATIVE_MARGIN_SCALE: 0.8,
+            # Dropout rate for encoder
+            DROP_RATE: 0.2,
+            # Dropout rate for attention
+            DROP_RATE_ATTENTION: 0,
+            # If 'True' apply dropout to sparse input tensors
+            SPARSE_INPUT_DROPOUT: False,
+            # If 'True' apply dropout to dense input tensors
+            DENSE_INPUT_DROPOUT: False,
+            # ## Evaluation parameters
+            # How often calculate validation accuracy.
+            # Small values may hurt performance, e.g. model accuracy.
+            EVAL_NUM_EPOCHS: 20,
+            # How many examples to use for hold out validation set
+            # Large values may hurt performance, e.g. model accuracy.
+            EVAL_NUM_EXAMPLES: 0,
+            # ## Selector config
+            # If 'True' random tokens of the input message will be masked and the model
+            # should predict those tokens.
+            MASKED_LM: False,
+            # Name of the intent for which this response selector is to be trained
+            RETRIEVAL_INTENT: None,
+            # Boolean flag to check if actual text of the response
+            # should be used as ground truth label for training the model.
+            USE_TEXT_AS_LABEL: False,
+            # If you want to use tensorboard to visualize training
+            # and validation metrics,
+            # set this option to a valid output directory.
+            TENSORBOARD_LOG_DIR: None,
+            # Define when training metrics for tensorboard should be logged.
+            # Either after every epoch or for every training step.
+            # Valid values: 'epoch' and 'batch'
+            TENSORBOARD_LOG_LEVEL: "epoch",
+            # Specify what features to use as sequence and sentence features
+            # By default all features in the pipeline are used.
+            FEATURIZERS: [],
+            # Perform model checkpointing
+            CHECKPOINT_MODEL: False,
+            # if 'True' applies sigmoid on all similarity terms and adds it
+            # to the loss function to ensure that similarity values are
+            # approximately bounded. Used inside cross-entropy loss only.
+            CONSTRAIN_SIMILARITIES: False,
+            # Model confidence to be returned during inference. Currently, the only
+            # possible value is `softmax`.
+            MODEL_CONFIDENCE: SOFTMAX,
+        }
 
     def __init__(
         self,
-        component_config: Optional[Dict[Text, Any]] = None,
+        config: Dict[Text, Any],
+        model_storage: ModelStorage,
+        resource: Resource,
+        execution_context: ExecutionContext,
         index_label_id_mapping: Optional[Dict[int, Text]] = None,
         entity_tag_specs: Optional[List[EntityTagSpec]] = None,
         model: Optional[RasaModel] = None,
         all_retrieval_intents: Optional[List[Text]] = None,
         responses: Optional[Dict[Text, List[Dict[Text, Any]]]] = None,
-        finetune_mode: bool = False,
         sparse_feature_sizes: Optional[Dict[Text, Dict[Text, List[int]]]] = None,
     ) -> None:
         """Declare instance variables with default values.
 
         Args:
-            component_config: Configuration for the component.
+            config: Configuration for the component.
+            model_storage: Storage which graph components can use to persist and load
+                themselves.
+            resource: Resource locator for this component which can be used to persist
+                and load itself from the `model_storage`.
+            execution_context: Information about the current graph run.
             index_label_id_mapping: Mapping between label and index used for encoding.
             entity_tag_specs: Format specification all entity tags.
             model: Model architecture.
@@ -273,7 +296,7 @@ class ResponseSelector(DIETClassifier):
                 otherwise initializes it with random weights.
             sparse_feature_sizes: Sizes of the sparse features the model was trained on.
         """
-        component_config = component_config or {}
+        component_config = config
 
         # the following properties cannot be adapted for the ResponseSelector
         component_config[INTENT_CLASSIFICATION] = True
@@ -288,31 +311,36 @@ class ResponseSelector(DIETClassifier):
 
         super().__init__(
             component_config,
+            model_storage,
+            resource,
+            execution_context,
             index_label_id_mapping,
             entity_tag_specs,
             model,
-            finetune_mode=finetune_mode,
             sparse_feature_sizes=sparse_feature_sizes,
         )
 
     @property
     def label_key(self) -> Text:
+        """Returns label key."""
         return LABEL_KEY
 
     @property
     def label_sub_key(self) -> Text:
+        """Returns label sub_key."""
         return LABEL_SUB_KEY
 
     @staticmethod
     def model_class(use_text_as_label: bool) -> Type[RasaModel]:
+        """Returns model class."""
         if use_text_as_label:
             return DIET2DIET
         else:
             return DIET2BOW
 
-    def _load_selector_params(self, config: Dict[Text, Any]) -> None:
-        self.retrieval_intent = config[RETRIEVAL_INTENT]
-        self.use_text_as_label = config[USE_TEXT_AS_LABEL]
+    def _load_selector_params(self) -> None:
+        self.retrieval_intent = self.component_config[RETRIEVAL_INTENT]
+        self.use_text_as_label = self.component_config[USE_TEXT_AS_LABEL]
 
     def _warn_about_transformer_and_hidden_layers_enabled(
         self, selector_name: Text
@@ -324,12 +352,13 @@ class ResponseSelector(DIETClassifier):
         then, from our experience, the best results are achieved with no hidden layers
         used between the feature-combining layers and the transformer.
         """
+        default_config = self.get_default_config()
         hidden_layers_is_at_default_value = (
             self.component_config[HIDDEN_LAYERS_SIZES]
-            == self.defaults[HIDDEN_LAYERS_SIZES]
+            == default_config[HIDDEN_LAYERS_SIZES]
         )
         config_for_disabling_hidden_layers = {
-            k: [] for k, _ in self.defaults[HIDDEN_LAYERS_SIZES].items()
+            k: [] for k, _ in default_config[HIDDEN_LAYERS_SIZES].items()
         }
         # warn if the hidden layers aren't disabled
         if (
@@ -368,13 +397,11 @@ class ResponseSelector(DIETClassifier):
                 f"`{self.component_config[TRANSFORMER_SIZE]}` for "
                 f"{selector_name}, but a positive size is required when using "
                 f"`{NUM_TRANSFORMER_LAYERS} > 0`. {selector_name} will proceed, using "
-                f"`{TRANSFORMER_SIZE}={self.default_transformer_size_when_enabled}`. "
+                f"`{TRANSFORMER_SIZE}={DEFAULT_TRANSFORMER_SIZE}`. "
                 f"Alternatively, specify a different value in the component's config.",
                 category=UserWarning,
             )
-            self.component_config[
-                TRANSFORMER_SIZE
-            ] = self.default_transformer_size_when_enabled
+            self.component_config[TRANSFORMER_SIZE] = DEFAULT_TRANSFORMER_SIZE
 
     def _check_config_params_when_transformer_enabled(self) -> None:
         """Checks & corrects config parameters when the transformer is enabled.
@@ -392,7 +419,7 @@ class ResponseSelector(DIETClassifier):
     def _check_config_parameters(self) -> None:
         """Checks that component configuration makes sense; corrects it where needed."""
         super()._check_config_parameters()
-        self._load_selector_params(self.component_config)
+        self._load_selector_params()
         # Once general DIET-related parameters have been checked, check also the ones
         # specific to ResponseSelector.
         self._check_config_params_when_transformer_enabled()
@@ -475,111 +502,117 @@ class ResponseSelector(DIETClassifier):
             It is always guaranteed to have a match, otherwise that case should have
             been caught earlier and a warning should have been raised.
         """
-
         for key, responses in self.responses.items():
 
             # First check if the predicted label was the key itself
             search_key = util.template_key_to_intent_response_key(key)
-            if hash(search_key) == label.get("id"):
+            if search_key == label.get("name"):
                 return search_key
 
             # Otherwise loop over the responses to check if the text has a direct match
             for response in responses:
-                if hash(response.get(TEXT, "")) == label.get("id"):
+                if response.get(TEXT, "") == label.get("name"):
                     return search_key
         return None
 
-    def process(self, message: Message, **kwargs: Any) -> None:
+    def process(self, messages: List[Message]) -> List[Message]:
         """Selects most like response for message.
 
         Args:
-            message: Latest user message.
-            kwargs: Additional key word arguments.
+            messages: List containing latest user message.
 
         Returns:
-            the most likely response, the associated intent_response_key and its
-            similarity to the input.
+            List containing the message augmented with the most likely response,
+            the associated intent_response_key and its similarity to the input.
         """
-        out = self._predict(message)
-        top_label, label_ranking = self._predict_label(out)
+        for message in messages:
+            out = self._predict(message)
+            top_label, label_ranking = self._predict_label(out)
 
-        # Get the exact intent_response_key and the associated
-        # responses for the top predicted label
-        label_intent_response_key = (
-            self._resolve_intent_response_key(top_label) or top_label[INTENT_NAME_KEY]
-        )
-        label_responses = self.responses.get(
-            util.intent_response_key_to_template_key(label_intent_response_key)
-        )
-
-        if label_intent_response_key and not label_responses:
-            # responses seem to be unavailable,
-            # likely an issue with the training data
-            # we'll use a fallback instead
-            rasa.shared.utils.io.raise_warning(
-                f"Unable to fetch responses for {label_intent_response_key} "
-                f"This means that there is likely an issue with the training data."
-                f"Please make sure you have added responses for this intent."
+            # Get the exact intent_response_key and the associated
+            # responses for the top predicted label
+            label_intent_response_key = (
+                self._resolve_intent_response_key(top_label)
+                or top_label[INTENT_NAME_KEY]
             )
-            label_responses = [{TEXT: label_intent_response_key}]
-
-        for label in label_ranking:
-            label[INTENT_RESPONSE_KEY] = (
-                self._resolve_intent_response_key(label) or label[INTENT_NAME_KEY]
+            label_responses = self.responses.get(
+                util.intent_response_key_to_template_key(label_intent_response_key)
             )
-            # Remove the "name" key since it is either the same as
-            # "intent_response_key" or it is the response text which
-            # is not needed in the ranking.
-            label.pop(INTENT_NAME_KEY)
 
-        selector_key = (
-            self.retrieval_intent
-            if self.retrieval_intent
-            else RESPONSE_SELECTOR_DEFAULT_INTENT
-        )
+            if label_intent_response_key and not label_responses:
+                # responses seem to be unavailable,
+                # likely an issue with the training data
+                # we'll use a fallback instead
+                rasa.shared.utils.io.raise_warning(
+                    f"Unable to fetch responses for {label_intent_response_key} "
+                    f"This means that there is likely an issue with the training data."
+                    f"Please make sure you have added responses for this intent."
+                )
+                label_responses = [{TEXT: label_intent_response_key}]
 
-        logger.debug(
-            f"Adding following selector key to message property: {selector_key}"
-        )
+            for label in label_ranking:
+                label[INTENT_RESPONSE_KEY] = (
+                    self._resolve_intent_response_key(label) or label[INTENT_NAME_KEY]
+                )
+                # Remove the "name" key since it is either the same as
+                # "intent_response_key" or it is the response text which
+                # is not needed in the ranking.
+                label.pop(INTENT_NAME_KEY)
 
-        # TODO: remove `RESPONSE_SELECTOR_RESPONSE_TEMPLATES_KEY` and
-        # `RESPONSE_SELECTOR_TEMPLATE_NAME_KEY` in Open Source 3.0.0
-        utter_action_key = util.intent_response_key_to_template_key(
-            label_intent_response_key
-        )
-        prediction_dict = {
-            RESPONSE_SELECTOR_PREDICTION_KEY: {
-                "id": top_label["id"],
-                RESPONSE_SELECTOR_RESPONSES_KEY: label_responses,
-                RESPONSE_SELECTOR_RESPONSE_TEMPLATES_KEY: label_responses,
-                PREDICTED_CONFIDENCE_KEY: top_label[PREDICTED_CONFIDENCE_KEY],
-                INTENT_RESPONSE_KEY: label_intent_response_key,
-                RESPONSE_SELECTOR_UTTER_ACTION_KEY: utter_action_key,
-                RESPONSE_SELECTOR_TEMPLATE_NAME_KEY: utter_action_key,
-            },
-            RESPONSE_SELECTOR_RANKING_KEY: label_ranking,
-        }
+            selector_key = (
+                self.retrieval_intent
+                if self.retrieval_intent
+                else RESPONSE_SELECTOR_DEFAULT_INTENT
+            )
 
-        self._set_message_property(message, prediction_dict, selector_key)
+            logger.debug(
+                f"Adding following selector key to message property: {selector_key}"
+            )
 
-        if out and DIAGNOSTIC_DATA in out:
-            message.add_diagnostic_data(self.unique_name, out.get(DIAGNOSTIC_DATA))
+            utter_action_key = util.intent_response_key_to_template_key(
+                label_intent_response_key
+            )
+            prediction_dict = {
+                RESPONSE_SELECTOR_PREDICTION_KEY: {
+                    RESPONSE_SELECTOR_RESPONSES_KEY: label_responses,
+                    PREDICTED_CONFIDENCE_KEY: top_label[PREDICTED_CONFIDENCE_KEY],
+                    INTENT_RESPONSE_KEY: label_intent_response_key,
+                    RESPONSE_SELECTOR_UTTER_ACTION_KEY: utter_action_key,
+                },
+                RESPONSE_SELECTOR_RANKING_KEY: label_ranking,
+            }
 
-    def persist(self, file_name: Text, model_dir: Text) -> Dict[Text, Any]:
-        """Persist this model into the passed directory.
+            self._set_message_property(message, prediction_dict, selector_key)
 
-        Return the metadata necessary to load the model again.
-        """
+            if (
+                self._execution_context.should_add_diagnostic_data
+                and out
+                and DIAGNOSTIC_DATA in out
+            ):
+                message.add_diagnostic_data(
+                    self._execution_context.node_name, out.get(DIAGNOSTIC_DATA)
+                )
+
+        return messages
+
+    def persist(self) -> None:
+        """Persist this model into the passed directory."""
         if self.model is None:
-            return {"file": None}
+            return None
 
-        super().persist(file_name, model_dir)
+        with self._model_storage.write_to(self._resource) as model_path:
+            file_name = self.__class__.__name__
 
-        return {
-            "file": file_name,
-            "responses": self.responses,
-            "all_retrieval_intents": self.all_retrieval_intents,
-        }
+            rasa.shared.utils.io.dump_obj_as_json_to_file(
+                model_path / f"{file_name}.responses.json", self.responses
+            )
+
+            rasa.shared.utils.io.dump_obj_as_json_to_file(
+                model_path / f"{file_name}.retrieval_intents.json",
+                self.all_retrieval_intents,
+            )
+
+        super().persist()
 
     @classmethod
     def _load_model_class(
@@ -588,7 +621,7 @@ class ResponseSelector(DIETClassifier):
         model_data_example: RasaModelData,
         label_data: RasaModelData,
         entity_tag_specs: List[EntityTagSpec],
-        meta: Dict[Text, Any],
+        config: Dict[Text, Any],
         finetune_mode: bool = False,
     ) -> "RasaModel":
 
@@ -600,19 +633,18 @@ class ResponseSelector(DIETClassifier):
                 if TEXT in feature_name
             },
         )
-        return cls.model_class(meta[USE_TEXT_AS_LABEL]).load(
+        return cls.model_class(config[USE_TEXT_AS_LABEL]).load(
             tf_model_file,
             model_data_example,
             predict_data_example,
             data_signature=model_data_example.get_signature(),
             label_data=label_data,
             entity_tag_specs=entity_tag_specs,
-            config=copy.deepcopy(meta),
+            config=copy.deepcopy(config),
             finetune_mode=finetune_mode,
         )
 
     def _instantiate_model_class(self, model_data: RasaModelData) -> "RasaModel":
-
         return self.model_class(self.use_text_as_label)(
             data_signature=model_data.get_signature(),
             label_data=self._label_data,
@@ -623,27 +655,40 @@ class ResponseSelector(DIETClassifier):
     @classmethod
     def load(
         cls,
-        meta: Dict[Text, Any],
-        model_dir: Text,
-        model_metadata: Metadata = None,
-        cached_component: Optional["ResponseSelector"] = None,
+        config: Dict[Text, Any],
+        model_storage: ModelStorage,
+        resource: Resource,
+        execution_context: ExecutionContext,
         **kwargs: Any,
-    ) -> "ResponseSelector":
+    ) -> ResponseSelector:
         """Loads the trained model from the provided directory."""
-
-        model = super().load(
-            meta, model_dir, model_metadata, cached_component, **kwargs
+        model: ResponseSelector = super().load(
+            config, model_storage, resource, execution_context, **kwargs
         )
-        if not meta.get("file"):
-            return model
 
-        model.responses = meta.get("responses", {})
-        model.all_retrieval_intents = meta.get("all_retrieval_intents", [])
-
-        return model
+        try:
+            with model_storage.read_from(resource) as model_path:
+                file_name = cls.__name__
+                responses = rasa.shared.utils.io.read_json_file(
+                    model_path / f"{file_name}.responses.json"
+                )
+                all_retrieval_intents = rasa.shared.utils.io.read_json_file(
+                    model_path / f"{file_name}.retrieval_intents.json"
+                )
+                model.responses = responses
+                model.all_retrieval_intents = all_retrieval_intents
+                return model
+        except ValueError:
+            logger.debug(
+                f"Failed to load {cls.__name__} from model storage. Resource "
+                f"'{resource.name}' doesn't exist."
+            )
+            return cls(config, model_storage, resource, execution_context)
 
 
 class DIET2BOW(DIET):
+    """DIET2BOW transformer implementation."""
+
     def _create_metrics(self) -> None:
         # self.metrics preserve order
         # output losses first
@@ -786,7 +831,7 @@ class DIET2DIET(DIET):
         #   effective sequence length of these features being 1 or 0.
         # We need to combine the two lengths to correctly get the last position.
         sentence_feature_lengths = self._get_sentence_feature_lengths(
-            self.tf_label_data, LABEL,
+            self.tf_label_data, LABEL
         )
         sentence_label = self._last_token(
             label_transformed, sequence_feature_lengths + sentence_feature_lengths
