@@ -593,7 +593,7 @@ def evaluate_intents(
     )
 
     report, precision, f1, accuracy, confusion_matrix, labels = _calculate_report(
-        output_directory, target_intents, predicted_intents, report_as_dict,
+        output_directory, target_intents, predicted_intents, report_as_dict
     )
     if output_directory:
         _dump_report(output_directory, "intent_report.json", report)
@@ -670,7 +670,7 @@ def _calculate_report(
         report_as_dict = bool(output_directory)
 
     report, precision, f1, accuracy = get_evaluation_metrics(
-        targets, predictions, output_dict=report_as_dict, exclude_label=exclude_label,
+        targets, predictions, output_dict=report_as_dict, exclude_label=exclude_label
     )
 
     if report_as_dict:
@@ -1382,7 +1382,7 @@ async def run_evaluation(
     from rasa.shared.constants import DEFAULT_DOMAIN_PATH
 
     test_data_importer = TrainingDataImporter.load_from_dict(
-        training_data_paths=[data_path], domain_path=DEFAULT_DOMAIN_PATH,
+        training_data_paths=[data_path], domain_path=DEFAULT_DOMAIN_PATH
     )
     test_data = test_data_importer.get_nlu_data()
 
@@ -1461,12 +1461,14 @@ def generate_folds(
                 training_examples=train,
                 entity_synonyms=training_data.entity_synonyms,
                 regex_features=training_data.regex_features,
+                lookup_tables=training_data.lookup_tables,
                 responses=training_data.responses,
             ),
             TrainingData(
                 training_examples=test,
                 entity_synonyms=training_data.entity_synonyms,
                 regex_features=training_data.regex_features,
+                lookup_tables=training_data.lookup_tables,
                 responses=training_data.responses,
             ),
         )
